@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -30,14 +30,29 @@ import subprocess
 import sys
 from logging import info, warning
 
+from nuitka.finalizations.FinalizeMarkups import getImportedNames
 from nuitka.importing import Importing, Recursion
 from nuitka.Options import getPythonFlags
 from nuitka.plugins.Plugins import Plugins
-from nuitka.PythonVersions import isUninstalledPython, python_version
+from nuitka.PythonVersions import (
+    getSupportedPythonVersions,
+    isStaticallyLinkedPython,
+    isUninstalledPython,
+    python_version,
+    python_version_str
+)
 from nuitka.tree import SyntaxErrors
 from nuitka.utils import Execution, InstanceCounters, MemoryUsage, Utils
+from nuitka.utils.AppDirs import getCacheDir
+from nuitka.utils.FileOperations import (
+    deleteFile,
+    hasFilenameExtension,
+    listDir,
+    makePath,
+    removeDirectory
+)
 
-from . import ModuleRegistry, Options, Tracing, TreeXML
+from . import ModuleRegistry, Options, TreeXML
 from .build import SconsInterface
 from .codegen import CodeGeneration, ConstantCodes
 from .finalizations import Finalization
@@ -70,16 +85,19 @@ def createNodeTree(filename):
     # harm.
     source_dir = getSourceDirectoryPath(main_module)
 
-    if not Options.shallOnlyExecCppCall():
+    if not Options.shallOnlyExecCCompilerCall():
         cleanSourceDirectory(source_dir)
 
     # Prepare the ".dist" directory, throwing away what was there before.
     if Options.isStandaloneMode():
         standalone_dir = getStandaloneDirectoryPath(main_module)
-        shutil.rmtree(standalone_dir, ignore_errors = True)
-        Utils.makePath(standalone_dir)
+        removeDirectory(
+            path          = standalone_dir,
+            ignore_errors = True
+        )
+        makePath(standalone_dir)
 
-    Utils.deleteFile(
+    deleteFile(
         path       = getResultFullpath(main_module),
         must_exist = False
     )
@@ -96,11 +114,44 @@ def createNodeTree(filename):
             pattern = pattern
         )
 
+    for package_name in Options.getMustIncludePackages():
+        package_package, package_directory, kind = Importing.findModule(
+            importing      = None,
+            module_name    = package_name,
+            parent_package = None,
+            level          = 0,
+            warn           = False
+        )
+
+        if kind != "absolute":
+            sys.exit("Error, failed to locate package %r." % package_name)
+
+        Recursion.checkPluginPath(
+            plugin_filename = package_directory,
+            module_package  = package_package
+        )
+
+    for module_name in Options.getMustIncludeModules():
+        module_package, module_filename, kind = Importing.findModule(
+            importing      = None,
+            module_name    = module_name,
+            parent_package = None,
+            level          = 0,
+            warn           = False
+        )
+
+        if kind != "absolute":
+            sys.exit("Error, failed to locate module %r." % module_name)
+
+        Recursion.checkPluginSinglePath(
+            plugin_filename = module_filename,
+            module_package  = module_package
+        )
 
     # Then optimize the tree and potentially recursed modules.
-    Optimization.optimize()
+    Optimization.optimize(main_module.getOutputFilename())
 
-    if Options.isExperimental():
+    if Options.isExperimental("check_xml_persistence"):
         for module in ModuleRegistry.getRootModules():
             if module.isMainModule():
                 return module
@@ -132,7 +183,7 @@ def getSourceDirectoryPath(main_module):
     assert main_module.isCompiledPythonModule()
 
     return Options.getOutputPath(
-        path = Utils.basename(
+        path = os.path.basename(
             getTreeFilenameWithSuffix(main_module, ".build")
         )
     )
@@ -140,7 +191,7 @@ def getSourceDirectoryPath(main_module):
 
 def getStandaloneDirectoryPath(main_module):
     return Options.getOutputPath(
-        path = Utils.basename(
+        path = os.path.basename(
             getTreeFilenameWithSuffix(main_module, ".dist")
         )
     )
@@ -150,15 +201,15 @@ def getResultBasepath(main_module):
     assert main_module.isCompiledPythonModule()
 
     if Options.isStandaloneMode():
-        return Utils.joinpath(
+        return os.path.join(
             getStandaloneDirectoryPath(main_module),
-            Utils.basename(
+            os.path.basename(
                 getTreeFilenameWithSuffix(main_module, "")
             )
         )
     else:
         return Options.getOutputPath(
-            path = Utils.basename(
+            path = os.path.basename(
                 getTreeFilenameWithSuffix(main_module, "")
             )
         )
@@ -176,14 +227,17 @@ def getResultFullpath(main_module):
 
 
 def cleanSourceDirectory(source_dir):
-    if Utils.isDir(source_dir):
-        for path, _filename in Utils.listDir(source_dir):
-            if Utils.getExtension(path) in (".c", ".h", ".o", ".os", ".obj",
-                                            ".bin", ".res", ".rc", ".S", ".cpp",
-                                            ".manifest"):
-                Utils.deleteFile(path, True)
+    if os.path.isdir(source_dir):
+        for path, _filename in listDir(source_dir):
+            if hasFilenameExtension(
+                path       = path,
+                extensions = (".c", ".h", ".o", ".os", ".obj",
+                              ".bin", ".res", ".rc", ".S", ".cpp",
+                              ".manifest")
+            ):
+                deleteFile(path, must_exist = True)
     else:
-        Utils.makePath(source_dir)
+        makePath(source_dir)
 
 
 def pickSourceFilenames(source_dir, modules):
@@ -194,7 +248,7 @@ def pickSourceFilenames(source_dir, modules):
     module_filenames = {}
 
     def getFilenames(module):
-        base_filename =  Utils.joinpath(
+        base_filename =  os.path.join(
             source_dir,
             "module." + module.getFullName()
               if not module.isInternalModule() else
@@ -204,7 +258,7 @@ def pickSourceFilenames(source_dir, modules):
         # Note: Could detect if the file system is cases sensitive in source_dir
         # or not, but that's probably not worth the effort. False positives do
         # no harm at all.
-        collision_filename = Utils.normcase(base_filename)
+        collision_filename = os.path.normcase(base_filename)
 
         return base_filename, collision_filename
 
@@ -253,14 +307,14 @@ def makeSourceDirectory(main_module):
 
     """
     # We deal with a lot of details here, but rather one by one, and split makes
-    # no sense, pylint: disable=R0912,R0914
+    # no sense, pylint: disable=too-many-branches,too-many-locals,too-many-statements
 
     assert main_module.isCompiledPythonModule()
 
     # The global context used to generate code.
     global_context = CodeGeneration.makeGlobalContext()
 
-    assert main_module in ModuleRegistry.getDoneModules()
+    # assert main_module in ModuleRegistry.getDoneModules()
 
     # We might have chosen to include it as bytecode, and only compiled it for
     # fun, and to find its imports. In this case, now we just can drop it. Or
@@ -280,7 +334,8 @@ def makeSourceDirectory(main_module):
                 # module. Some uncompiled modules may have been asked by the user
                 # or technically required. By default, frozen code if it exists
                 # is preferred, as it will be from standalone mode adding it.
-                if uncompiled_module.isUserProvided():
+                if uncompiled_module.isUserProvided() or \
+                   uncompiled_module.isTechnical():
                     ModuleRegistry.removeDoneModule(module)
                 else:
                     ModuleRegistry.removeUncompiledModule(uncompiled_module)
@@ -288,6 +343,9 @@ def makeSourceDirectory(main_module):
     # Lets check if the recurse-to modules are actually present, and warn the
     # user if one of those was not found.
     for any_case_module in Options.getShallFollowModules():
+        if '*' in any_case_module or '{' in any_case_module:
+            continue
+
         for module in ModuleRegistry.getDoneUserModules():
             if module.getFullName() == any_case_module:
                 break
@@ -317,9 +375,9 @@ def makeSourceDirectory(main_module):
 
     for module in ModuleRegistry.getDoneModules():
         if module.isCompiledPythonModule():
-            cpp_filename = module_filenames[module]
+            c_filename = module_filenames[module]
 
-            prepared_modules[cpp_filename] = CodeGeneration.prepareModuleCode(
+            prepared_modules[c_filename] = CodeGeneration.prepareModuleCode(
                 global_context = global_context,
                 module         = module,
                 module_name    = module.getFullName(),
@@ -327,14 +385,14 @@ def makeSourceDirectory(main_module):
 
             # Main code constants need to be allocated already too.
             if module is main_module and not Options.shallMakeModule():
-                prepared_modules[cpp_filename][1].getConstantCode(0)
+                prepared_modules[c_filename][1].getConstantCode(0)
 
     # Second pass, generate the actual module code into the files.
     for module in ModuleRegistry.getDoneModules():
         if module.isCompiledPythonModule():
-            cpp_filename = module_filenames[module]
+            c_filename = module_filenames[module]
 
-            template_values, module_context = prepared_modules[cpp_filename]
+            template_values, module_context = prepared_modules[c_filename]
 
             source_code = CodeGeneration.generateModuleCode(
                 module_context  = module_context,
@@ -342,14 +400,14 @@ def makeSourceDirectory(main_module):
             )
 
             writeSourceCode(
-                filename    = cpp_filename,
+                filename    = c_filename,
                 source_code = source_code
             )
 
             if Options.isShowInclusion():
                 info("Included compiled module '%s'." % module.getFullName())
         elif module.isPythonShlibModule():
-            target_filename = Utils.joinpath(
+            target_filename = os.path.join(
                 getStandaloneDirectoryPath(main_module),
                 *module.getFullName().split('.')
             )
@@ -359,18 +417,22 @@ def makeSourceDirectory(main_module):
             else:
                 target_filename += ".so"
 
-            target_dir = Utils.dirname(target_filename)
+            target_dir = os.path.dirname(target_filename)
 
-            if not Utils.isDir(target_dir):
-                Utils.makePath(target_dir)
+            if not os.path.isdir(target_dir):
+                makePath(target_dir)
 
-            shutil.copy(
+            shutil.copyfile(
                 module.getFilename(),
                 target_filename
             )
 
             standalone_entry_points.append(
-                (target_filename, module.getPackage())
+                (
+                    module.getFilename(),
+                    target_filename,
+                    module.getPackage()
+                )
             )
         elif module.isUncompiledPythonModule():
             pass
@@ -378,7 +440,7 @@ def makeSourceDirectory(main_module):
             assert False, module
 
     writeSourceCode(
-        filename    = Utils.joinpath(
+        filename    = os.path.join(
             source_dir,
             "__constants.c"
         ),
@@ -392,53 +454,58 @@ def makeSourceDirectory(main_module):
     )
 
     writeSourceCode(
-        filename    = Utils.joinpath(source_dir, "__helpers.h"),
+        filename    = os.path.join(
+            source_dir,
+            "__helpers.h"
+        ),
         source_code = helper_decl_code
     )
 
     writeSourceCode(
-        filename    = Utils.joinpath(source_dir, "__helpers.c"),
+        filename    = os.path.join(
+            source_dir,
+            "__helpers.c"
+        ),
         source_code = helper_impl_code
     )
 
 
+def _asBoolStr(value):
+    return "true" if value else "false"
+
+
 def runScons(main_module, quiet):
     # Scons gets transported many details, that we express as variables, and
-    # have checks for them, leading to many branches, pylint: disable=R0912
-
-    python_version_str = "%d.%d" % (sys.version_info[0], sys.version_info[1])
+    # have checks for them, leading to many branches, pylint: disable=too-many-branches,too-many-statements
 
     if hasattr(sys, "abiflags"):
+        abiflags = sys.abiflags
+
         if Options.isPythonDebug() or \
            hasattr(sys, "getobjects"):
-            if sys.abiflags.startswith('d'):
-                python_version_str += sys.abiflags
-            else:
-                python_version_str += 'd' + sys.abiflags
-        else:
-            python_version_str += sys.abiflags
-
-    def asBoolStr(value):
-        return "true" if value else "false"
+            if not abiflags.startswith('d'):
+                abiflags = 'd' + abiflags
+    else:
+        abiflags = None
 
     options = {
-        "name"            : Utils.basename(
+        "name"            : os.path.basename(
             getTreeFilenameWithSuffix(main_module, "")
         ),
         "result_name"     : getResultBasepath(main_module),
         "source_dir"      : getSourceDirectoryPath(main_module),
-        "debug_mode"      : asBoolStr(Options.isDebug()),
-        "python_debug"    : asBoolStr(Options.isPythonDebug()),
-        "unstripped_mode" : asBoolStr(Options.isUnstripped()),
-        "module_mode"     : asBoolStr(Options.shallMakeModule()),
-        "optimize_mode"   : asBoolStr(Options.isOptimize()),
-        "full_compat"     : asBoolStr(Options.isFullCompat()),
-        "experimental"    : asBoolStr(Options.isExperimental()),
-        "trace_mode"      : asBoolStr(Options.shallTraceExecution()),
+        "debug_mode"      : _asBoolStr(Options.isDebug()),
+        "python_debug"    : _asBoolStr(Options.isPythonDebug()),
+        "unstripped_mode" : _asBoolStr(Options.isUnstripped()),
+        "module_mode"     : _asBoolStr(Options.shallMakeModule()),
+        "full_compat"     : _asBoolStr(Options.isFullCompat()),
+        "experimental"    : ','.join(Options.getExperimentalIndications()),
+        "trace_mode"      : _asBoolStr(Options.shallTraceExecution()),
         "python_version"  : python_version_str,
         "target_arch"     : Utils.getArchitecture(),
         "python_prefix"   : sys.prefix,
         "nuitka_src"      : SconsInterface.getSconsDataPath(),
+        "nuitka_cache"    : getCacheDir(),
         "module_count"    : "%d" % (
             1 + \
             len(ModuleRegistry.getDoneUserModules()) + \
@@ -465,6 +532,9 @@ def runScons(main_module, quiet):
        isUninstalledPython():
         options["uninstalled_python"] = "true"
 
+    if isStaticallyLinkedPython():
+        options["static_python"] = "true"
+
     if ModuleRegistry.getUncompiledTechnicalModules():
         options["frozen_modules"] = str(
             len(ModuleRegistry.getUncompiledTechnicalModules())
@@ -515,6 +585,11 @@ def runScons(main_module, quiet):
     if python_version < 300 and sys.flags.unicode:
         options["python_sysflag_unicode"] = "true"
 
+    if python_version >= 370 and sys.flags.utf8_mode:
+        options["python_sysflag_utf8"] = "true"
+
+    if abiflags:
+        options["abiflags"] = abiflags
 
     return SconsInterface.runScons(options, quiet), options
 
@@ -522,7 +597,7 @@ def runScons(main_module, quiet):
 def writeSourceCode(filename, source_code):
     # Prevent accidental overwriting. When this happens the collision detection
     # or something else has failed.
-    assert not Utils.isFile(filename), filename
+    assert not os.path.isfile(filename), filename
 
     if python_version >= 300:
         with open(filename, "wb") as output_file:
@@ -535,7 +610,7 @@ def writeSourceCode(filename, source_code):
 def writeBinaryData(filename, binary_data):
     # Prevent accidental overwriting. When this happens the collision detection
     # or something else has failed.
-    assert not Utils.isFile(filename), filename
+    assert not os.path.isfile(filename), filename
 
     assert type(binary_data) is bytes
 
@@ -603,7 +678,7 @@ def executeModule(tree, clean_path):
 def compileTree(main_module):
     source_dir = getSourceDirectoryPath(main_module)
 
-    if not Options.shallOnlyExecCppCall():
+    if not Options.shallOnlyExecCCompilerCall():
         # Now build the target language code for the whole tree.
         makeSourceDirectory(
             main_module = main_module
@@ -613,7 +688,7 @@ def compileTree(main_module):
 
         if frozen_code is not None:
             writeSourceCode(
-                filename    = Utils.joinpath(
+                filename    = os.path.join(
                     source_dir,
                     "__frozen.c"
                 ),
@@ -621,17 +696,20 @@ def compileTree(main_module):
             )
 
         writeBinaryData(
-            filename    = Utils.joinpath(source_dir, "__constants.bin"),
+            filename    = os.path.join(
+                source_dir,
+                "__constants.bin"
+            ),
             binary_data = ConstantCodes.stream_data.getBytes()
         )
     else:
         source_dir = getSourceDirectoryPath(main_module)
 
-        if not Utils.isFile(Utils.joinpath(source_dir, "__helpers.h")):
+        if not os.path.isfile(os.path.join(source_dir, "__helpers.h")):
             sys.exit("Error, no previous build directory exists.")
 
     if Options.isShowProgress() or Options.isShowMemory():
-        Tracing.printLine(
+        info(
             "Total memory usage before running scons: {memory}:".format(
                 memory = MemoryUsage.getHumanReadableProcessMemoryUsage()
             )
@@ -640,7 +718,7 @@ def compileTree(main_module):
     if Options.isShowMemory():
         InstanceCounters.printStats()
 
-    if Options.shallNotDoExecCppCall():
+    if Options.shallNotDoExecCCompilerCall():
         return True, {}
 
     # Run the Scons to build things.
@@ -652,6 +730,30 @@ def compileTree(main_module):
     return result, options
 
 
+def handleSyntaxError(e):
+    # Syntax or indentation errors, output them to the user and abort. If
+    # we are not in full compat, and user has not specified the Python
+    # versions he wants, tell him about the potential version problem.
+    error_message = SyntaxErrors.formatOutput(e)
+
+    if not Options.isFullCompat() and \
+       Options.getIntendedPythonVersion() is None:
+        if python_version < 300:
+            suggested_python_version_str = getSupportedPythonVersions()[-1]
+        else:
+            suggested_python_version_str = "2.7"
+
+        error_message += """
+
+Nuitka is very syntax compatible with standard Python. It is currently running
+with Python version '%s', you might want to specify more clearly with the use
+of the precise Python interpreter binary and '-m nuitka', e.g. use this
+'python%s -m nuitka' option, if that's not the one the program expects.
+""" % (python_version_str, suggested_python_version_str)
+
+    sys.exit(error_message)
+
+
 data_files = []
 
 def main():
@@ -661,22 +763,18 @@ def main():
         in the desired version of Python with desired flags, and we just get
         to execute the task assigned.
 
-        We might be asked to only re-compile generated C++, dump only an XML
+        We might be asked to only re-compile generated C, dump only an XML
         representation of the internal node tree after optimization, etc.
     """
 
-    # Main has to fullfil many options, leading to many branches and statements
-    # to deal with them.  pylint: disable=R0912
-
-    positional_args = Options.getPositionalArgs()
-    assert len(positional_args) > 0
-
+    # Main has to fulfill many options, leading to many branches and statements
+    # to deal with them.  pylint: disable=too-many-branches
     filename = Options.getPositionalArgs()[0]
 
     # Inform the importing layer about the main script directory, so it can use
     # it when attempting to follow imports.
     Importing.setMainScriptDirectory(
-        main_dir = Utils.dirname(Utils.abspath(filename))
+        main_dir = os.path.dirname(os.path.abspath(filename))
     )
 
     # Detect to be frozen modules if any, so we can consider to not recurse
@@ -686,12 +784,12 @@ def main():
             ModuleRegistry.addUncompiledModule(module)
 
             if module.getName() == "site":
-                origin_prefix_filename = Utils.joinpath(
-                    Utils.dirname(module.getCompileTimeFilename()),
+                origin_prefix_filename = os.path.join(
+                    os.path.dirname(module.getCompileTimeFilename()),
                     "orig-prefix.txt"
                 )
 
-                if Utils.isFile(origin_prefix_filename):
+                if os.path.isfile(origin_prefix_filename):
                     data_files.append(
                         (filename, "orig-prefix.txt")
                     )
@@ -702,17 +800,17 @@ def main():
             filename = filename
         )
     except (SyntaxError, IndentationError) as e:
-        # Syntax or indentation errors, output them to the user and abort.
-        sys.exit(
-            SyntaxErrors.formatOutput(e)
-        )
+        handleSyntaxError(e)
 
     if Options.shallDumpBuiltTreeXML():
+        # XML only.
         for module in ModuleRegistry.getDoneModules():
             dumpTreeXML(module)
     elif Options.shallDisplayBuiltTree():
+        # GUI only.
         displayTree(main_module)
     else:
+        # Make the actual compilation.
         result, options = compileTree(
             main_module = main_module
         )
@@ -721,21 +819,18 @@ def main():
         if not result:
             sys.exit(1)
 
-        if Options.shallNotDoExecCppCall():
-            sys.exit(0)
+        if Options.shallNotDoExecCCompilerCall():
+            if Options.isShowMemory():
+                MemoryUsage.showMemoryTrace()
 
-        # Remove the source directory (now build directory too) if asked to.
-        if Options.isRemoveBuildDir():
-            shutil.rmtree(
-                getSourceDirectoryPath(main_module)
-            )
+            sys.exit(0)
 
         if Options.isStandaloneMode():
             binary_filename = options["result_name"] + ".exe"
 
             standalone_entry_points.insert(
                 0,
-                (binary_filename, None)
+                (binary_filename, binary_filename, None)
             )
 
             dist_dir = getStandaloneDirectoryPath(main_module)
@@ -751,29 +846,36 @@ def main():
                 )
 
             copyUsedDLLs(
+                source_dir              = getSourceDirectoryPath(main_module),
                 dist_dir                = dist_dir,
                 standalone_entry_points = standalone_entry_points
             )
 
-
             for module in ModuleRegistry.getDoneModules():
                 data_files.extend(
                     Plugins.considerDataFiles(module)
                 )
 
             for source_filename, target_filename in data_files:
-                target_filename = Utils.joinpath(
+                target_filename = os.path.join(
                     getStandaloneDirectoryPath(main_module),
                     target_filename
                 )
 
-                Utils.makePath(Utils.dirname(target_filename))
+                makePath(os.path.dirname(target_filename))
 
                 shutil.copy2(
                     source_filename,
                     target_filename
                 )
 
+        # Remove the source directory (now build directory too) if asked to.
+        if Options.isRemoveBuildDir():
+            removeDirectory(
+                path          = getSourceDirectoryPath(main_module),
+                ignore_errors = False
+            )
+
         # Modules should not be executable, but Scons creates them like it, fix
         # it up here.
         if Utils.getOS() != "Windows" and Options.shallMakeModule():
@@ -785,6 +887,41 @@ def main():
                 )
             )
 
+        if Options.shallMakeModule() and Options.shallCreatePyiFile():
+            pyi_filename = getResultBasepath(main_module) + ".pyi"
+
+            with open(pyi_filename, 'w') as pyi_file:
+                pyi_file.write(
+                    """\
+# This file was generated by Nuitka and describes the types of the
+# created shared library.
+
+# At this time it lists only the imports made and can be used by the
+# tools that bundle libraries, including Nuitka itself. For instance
+# standalone mode usage of the created library will need it.
+
+# In the future, this will also contain type information for values
+# in the module, so IDEs will use this. Therfore please include it
+# when you make software releases of the extension module that it
+# describes.
+
+%(imports)s
+
+# This is not Python source even if it looks so. Make it clear for
+# now. This was decided by PEP 484 designers.
+__name__ = ...
+
+""" % {
+                        "imports" : '\n'.join(
+                            "import %s" % module_name
+                            for module_name in
+                            getImportedNames()
+                        )
+
+                    }
+                )
+
+
         # Execute the module immediately if option was given.
         if Options.shallExecuteImmediately():
             if Options.shallMakeModule():
